# -*- coding: utf-8 -*-
"""
Expose exactly one plugin config class at import time so straight.plugin can
discover it without loading both variants (which can cause duplicate app labels).
"""

from ocyan.plugin.oscar.info import OSCAR_NEEDS_OXYAN

if OSCAR_NEEDS_OXYAN:
    from .entrypoint import Oscar3ElasticsearchConfig as OscarElasticsearchConfig
else:
    from .entrypoint import OscarElasticsearchConfig

__all__ = ["OscarElasticsearchConfig"]
