# -*- coding: utf-8 -*-
from django.apps import AppConfig


class OcyanPluginOscarElasticsearchAppConfig(AppConfig):
    name = "ocyan.plugin.oscar_elasticsearch"
    label = "ocyan_plugin_oscar_elasticsearch"
