"""
Use the Oscar Elasticsearch backend client for catalogue/search operations.

This keeps query and indexing paths on the same client/index settings.
"""

from oscar_elasticsearch.search.backend import es
