from oscar_elasticsearch.search import apps


class OscarElasticSearchConfig(apps.OscarElasticSearchConfig):
    name = "ocyan.plugin.oscar_elasticsearch.search"

    def ready(self):
        super().ready()
        # pylint: disable=unused-import
        from . import search_signal_handlers, signal_handlers

    def get_urls(self):
        from .urls import urlpatterns

        return super().get_urls() + urlpatterns
