from ocyan.main.loading import get_baseclasses

# pylint: disable=unbalanced-tuple-unpacking
BaseCatalogueView, BaseProductCategoryView = get_baseclasses(
    "catalogue.views",
    ["CatalogueView", "ProductCategoryView"],
    "ocyan.plugin.oscar_elasticsearch.catalogue_es_extension",
)


class CatalogueView(BaseCatalogueView):
    order_by_relevance = False


class ProductCategoryView(BaseProductCategoryView):
    order_by_relevance = False
