from django.urls import path
from django.views.generic import TemplateView

from .views import OpenSearchAutocompleteView, update_oscar_index

urlpatterns = [
    path(
        "opensearchdescription.xml",
        TemplateView.as_view(
            template_name="oscar/search/opensearchdescription.xml",
            content_type="application/opensearchdescription+xml",
        ),
        name="opensearchdescription",
    ),
    path(
        "opensearch-autocomplete/",
        OpenSearchAutocompleteView.as_view(),
        name="opensearch-autocomplete",
    ),
    path("update_oscar_index/", update_oscar_index, name="update_oscar_index"),
]
